//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : Collatz
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   The Collatz conjecture states: Any positive number will be eventually
// be reduced to 1 by repeating the following process
//
//   o If a number is even, divide it by 2
//   o If a number is odd,  multiply it by 3 and add 1
//     (the number will now be even, so it will next be divided by 2)
//
//   This program prompts the user for a legal starting value and implements
// the Collatz process, displaying the sequence of values guenerated by the
// process. When the number is reduced to 1, the program reports the total
// number of cycles it took (how many times it applied the process).
//
//   This program uses the BigInteger class, which slows it down (compared
// to using int) but removes any limit on number of digits. BigInteger
// objects are constructed from Strings (representing numbers). There
// are no operators on these objects: we must use BigInteger methods to
// perform arithmetic (add, multiply, divide, remainder, comparisons
// (equals), etc. See the original Collatz program and the changes required
// here. Note that ZERO and ONE are static values supplied by the class.
//
//   This program also uses a Timer object to time the speed of each
// Collatz process. The user is given an option to see intermediate
// results.
//
// Known Bugs     : None
//
// Future Plans   : Ensure user enters a positive value initially
//
// Program History:
//   5/18/01: R. Pattis - Operational for 15-100
//   1/25/02: R. Pattis - Added Timer object for timing process
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//Imports for my classes (which must be added to the project)
import edu.cmu.cs.pattis.cs151xx.Prompt;
import edu.cmu.cs.pattis.cs151xx.Timer;

//Imports for Java classes
import java.math.BigInteger;



public class Application {

  
	public static void main(String args[])
  {
    final BigInteger TWO        = new BigInteger("2");
    final BigInteger THREE      = new BigInteger("3");
      
    BigInteger originalNumber = Prompt.forBigInteger("Enter a positive integer     ");
    boolean    isDebugging    = Prompt.forBoolean   ("Display intermediate results?");

    int        cycleCount     = 1;
    BigInteger testNumber     = originalNumber;


    Timer processTimer = new Timer();


    //Implement Collatz process (as described above)

    processTimer.start();                                         //Start timer after all prompting
    for(;;) {
      //Display current status
      if (isDebugging)
        System.out.println("Cycle " + cycleCount + ": testNumber is now " + testNumber);

      //////////////////////////////////////
      if (testNumber.equals(BigInteger.ONE))                      //Check for termination
        break;
      //////////////////////////////////////

      //Update status (no possible overflow with BigIntegers)
      cycleCount++;
      if (testNumber.remainder(TWO).equals(BigInteger.ZERO))      //Is even if remainder after /2 is 0
        testNumber = testNumber.divide(TWO);
      else  
        testNumber = testNumber.multiply(THREE).add(BigInteger.ONE);
    }
    processTimer.stop();                                          //Could omit and just call getElapsed
  
  
    //Display statistics gathered during Collatz process
  
    System.out.println("\n\nFor Test Number = " + originalNumber + ", Cycles to 1 = " + cycleCount + "\n");
    System.out.println("Process took " + processTimer.getElapsed() + " seconds");

  }
}
